@@ -19,6 +19,10 @@
  *       - 248293: JPA 2.0 Element Collections (part 1)
  *     03/27/2009-2.0 Guy Pelletier 
  *       - 241413: JPA 2.0 Add EclipseLink support for Map type attributes
+ *     03/08/2010-2.1 Guy Pelletier 
+ *       - 303632: Add attribute-type for mapping attributes to EclipseLink-ORM
+ *     05/14/2010-2.1 Guy Pelletier 
+ *       - 253083: Add support for dynamic persistence using ORM.xml/eclipselink-orm.xml
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.metadata;
 
@@ -114,6 +118,15 @@ public class MetadataProcessor {
         m_project.processQueries(m_loader);
     }
     
+    /**
+     * INTERNAL:
+     * During EntityManagerSetup deploy, using the real class loader we must
+     * create our dynamic classes.
+     */
+    public void createDynamicClasses() {
+        m_project.createDynamicClasses(m_loader);
+    }
+    
     /**
      * INTERNAL:
      */
@@ -132,15 +145,15 @@ public class MetadataProcessor {
         
         for (XMLEntityMappings entityMappings : m_project.getEntityMappings()) {
             for (ClassAccessor entity : entityMappings.getEntities()) {
-                classSet.add(entityMappings.getFullyQualifiedClassName(entity.getClassName()));
+                classSet.add(entityMappings.getPackageQualifiedClassName(entity.getClassName()));
             }
             
             for (ClassAccessor embeddable : entityMappings.getEmbeddables()) {
-                classSet.add(entityMappings.getFullyQualifiedClassName(embeddable.getClassName()));
+                classSet.add(entityMappings.getPackageQualifiedClassName(embeddable.getClassName()));
             }
             
             for (ClassAccessor mappedSuperclass : entityMappings.getMappedSuperclasses()) {
-                classSet.add(entityMappings.getFullyQualifiedClassName(mappedSuperclass.getClassName()));
+                classSet.add(entityMappings.getPackageQualifiedClassName(mappedSuperclass.getClassName()));
             }
         }
         
@@ -297,7 +310,7 @@ public class MetadataProcessor {
         // metadata where necessary. Note: we want the eclipselink orm
         // metadata to merge into standard jpa files and not vice versa. 
         // Loading them last will ensure this happens.
-        Boolean excludeEclipseLinkORM = new Boolean((String) m_project.getPersistenceUnitInfo().getProperties().get(PersistenceUnitProperties.EXCLUDE_ECLIPSELINK_ORM_FILE));
+        Boolean excludeEclipseLinkORM = Boolean.valueOf((String) m_project.getPersistenceUnitInfo().getProperties().get(PersistenceUnitProperties.EXCLUDE_ECLIPSELINK_ORM_FILE));
         if (! excludeEclipseLinkORM) {
             loadStandardMappingFiles(MetadataHelper.ECLIPSELINK_ORM_FILE);
         }
