@@ -27,6 +27,16 @@
  *       - 270011: JPA 2.0 MappedById support
  *     09/29/2009-2.0 Guy Pelletier 
  *       - 282553: JPA 2.0 JoinTable support for OneToOne and ManyToOne
+ *     03/08/2010-2.1 Guy Pelletier 
+ *       - 303632: Add attribute-type for mapping attributes to EclipseLink-ORM
+ *     03/29/2010-2.1 Guy Pelletier 
+ *       - 267217: Add Named Access Type to EclipseLink-ORM
+ *     04/09/2010-2.1 Guy Pelletier 
+ *       - 307050: Add defaults for access methods of a VIRTUAL access type
+ *     04/27/2010-2.1 Guy Pelletier 
+ *       - 309856: MappedSuperclasses from XML are not being initialized properly
+ *     05/14/2010-2.1 Guy Pelletier 
+ *       - 253083: Add support for dynamic persistence using ORM.xml/eclipselink-orm.xml
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.metadata.accessors;
 
@@ -57,6 +67,7 @@ import org.eclipse.persistence.internal.jpa.metadata.converters.ObjectTypeConver
 import org.eclipse.persistence.internal.jpa.metadata.converters.StructConverterMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.converters.TypeConverterMetadata;
 
+import org.eclipse.persistence.internal.jpa.metadata.mappings.AccessMethodsMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.tables.TableMetadata;
 import org.eclipse.persistence.internal.jpa.metadata.xml.XMLEntityMappings;
 
@@ -74,7 +85,9 @@ import org.eclipse.persistence.internal.jpa.metadata.ORMetadata;
  * @since TopLink EJB 3.0 Reference Implementation
  */
 public abstract class MetadataAccessor extends ORMetadata {
-    private String m_access;
+    // Note: Any metadata mapped from XML to this class must be compared in the equals method.
+    
+    private AccessMethodsMetadata m_accessMethods;
     
     private List<ConverterMetadata> m_converters = new ArrayList<ConverterMetadata>();
     private List<ObjectTypeConverterMetadata> m_objectTypeConverters = new ArrayList<ObjectTypeConverterMetadata>();
@@ -84,6 +97,8 @@ public abstract class MetadataAccessor extends ORMetadata {
     
     private MetadataDescriptor m_descriptor;
     private MetadataProject m_project;
+    
+    private String m_access;
     private String m_name;
     
     /**
@@ -107,6 +122,48 @@ public abstract class MetadataAccessor extends ORMetadata {
         initAccess();
     }
     
+    /**
+     * INTERNAL:
+     */
+    @Override
+    public boolean equals(Object objectToCompare) {
+        if (objectToCompare instanceof MetadataAccessor) {
+            MetadataAccessor accessor = (MetadataAccessor) objectToCompare;
+                        
+            if (! valuesMatch(m_accessMethods, accessor.getAccessMethods())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_converters, accessor.getConverters())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_objectTypeConverters, accessor.getObjectTypeConverters())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_structConverters, accessor.getStructConverters())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_typeConverters, accessor.getTypeConverters())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_properties, accessor.getProperties())) {
+                return false;
+            }
+           
+            if (! valuesMatch(m_access, accessor.getAccess())) {
+                return false;
+            }
+            
+            return valuesMatch(m_name, accessor.getName());
+        }
+        
+        return false;
+    }
+    
     /**
      * INTERNAL:
      * Process and add the globally defined converters to the project.
@@ -150,6 +207,14 @@ public abstract class MetadataAccessor extends ORMetadata {
         return getAccessibleObject().getName();
     }
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public AccessMethodsMetadata getAccessMethods() {
+        return m_accessMethods;
+    }
+    
     /**
      * INTERNAL:
      * Return the annotated element for this accessor.
@@ -205,7 +270,7 @@ public abstract class MetadataAccessor extends ORMetadata {
     public MetadataDescriptor getDescriptor() {
         return m_descriptor;
     }
-
+    
     /**
      * INTERNAL:
      * To satisfy the abstract getIdentifier() method from ORMetadata.
@@ -359,8 +424,8 @@ public abstract class MetadataAccessor extends ORMetadata {
     /**
      * INTERNAL:
      */
-    public boolean hasPropertyAccess() {
-        return hasAccess() && m_access.equals(MetadataConstants.PROPERTY);
+    public boolean hasAccessMethods() {
+        return m_accessMethods != null;
     }
     
     /**
@@ -395,6 +460,9 @@ public abstract class MetadataAccessor extends ORMetadata {
     public void initXMLObject(MetadataAccessibleObject accessibleObject, XMLEntityMappings entityMappings) {
         super.initXMLObject(accessibleObject, entityMappings);
         
+        // Initialize single objects.
+        initXMLObject(m_accessMethods, accessibleObject);
+        
         // Initialize lists of objects.
         initXMLObjects(m_converters, accessibleObject);
         initXMLObjects(m_objectTypeConverters, accessibleObject);
@@ -430,6 +498,9 @@ public abstract class MetadataAccessor extends ORMetadata {
         // Simple object merging.
         m_access = (String) mergeSimpleObjects(m_access, accessor.getAccess(), accessor, "@access");
         
+        // ORMetadata object merging.        
+        m_accessMethods = (AccessMethodsMetadata) mergeORObjects(m_accessMethods, accessor.getAccessMethods());
+        
         // ORMetadata list merging.
         m_converters = mergeORObjectLists(m_converters, accessor.getConverters());
         m_objectTypeConverters = mergeORObjectLists(m_objectTypeConverters, accessor.getObjectTypeConverters());
@@ -563,6 +634,14 @@ public abstract class MetadataAccessor extends ORMetadata {
         m_access = access;
     } 
     
+    /**
+     * INTERNAL:
+     * Used for OX mapping.
+     */
+    public void setAccessMethods(AccessMethodsMetadata accessMethods){
+        m_accessMethods = accessMethods;
+    }
+    
     /**
      * INTERNAL:
      * Used for OX mapping.
@@ -593,6 +672,9 @@ public abstract class MetadataAccessor extends ORMetadata {
         // point if the global flag is set to false. 
         if (useDelimitedIdentifier()) {
             field.setUseDelimiters(useDelimitedIdentifier());
+        } else if (m_project.getShouldForceFieldNamesToUpperCase() && !field.shouldUseDelimiters()) {
+            //done directly as this field's name should be in uppercase.
+            field.setName(field.getName().toUpperCase());
         }
     }
     
