@@ -33,6 +33,10 @@
  *       - 290567: mappedbyid support incomplete
  *     11/23/2009-2.0 Guy Pelletier 
  *       - 295790: JPA 2.0 adding @MapsId to one entity causes initialization errors in other entities
+ *     03/29/2010-2.1 Guy Pelletier 
+ *       - 267217: Add Named Access Type to EclipseLink-ORM
+ *     04/27/2010-2.1 Guy Pelletier 
+ *       - 309856: MappedSuperclasses from XML are not being initialized properly
  ******************************************************************************/  
 package org.eclipse.persistence.internal.jpa.metadata.accessors.mappings;
 
@@ -68,6 +72,7 @@ import org.eclipse.persistence.internal.indirection.WeavedObjectBasicIndirection
 import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.mappings.DirectToFieldMapping;
 import org.eclipse.persistence.mappings.EmbeddableMapping;
+import org.eclipse.persistence.mappings.ManyToOneMapping;
 import org.eclipse.persistence.mappings.ObjectReferenceMapping;
 import org.eclipse.persistence.mappings.OneToOneMapping;
 import org.eclipse.persistence.mappings.RelationTableMechanism;
@@ -80,8 +85,10 @@ import org.eclipse.persistence.mappings.RelationTableMechanism;
  * @since TopLink EJB 3.0 Reference Implementation
  */
 public abstract class ObjectAccessor extends RelationshipAccessor {
+    // Note: Any metadata mapped from XML to this class must be compared in the equals method.
+
     private Boolean m_id;
-    private Boolean m_isOptional;
+    private Boolean m_optional;
     private List<PrimaryKeyJoinColumnMetadata> m_primaryKeyJoinColumns = new ArrayList<PrimaryKeyJoinColumnMetadata>();
     private String m_mapsId;
     
@@ -100,7 +107,7 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
         super(annotation, accessibleObject, classAccessor);
         
         if (annotation != null) {
-            m_isOptional = (Boolean) annotation.getAttribute("optional");
+            m_optional = (Boolean) annotation.getAttribute("optional");
         }
         
         // Set the primary key join columns if some are present.
@@ -130,6 +137,32 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
         m_id = isAnnotationPresent(Id.class);
     }
     
+    /**
+     * INTERNAL:
+     */
+    @Override
+    public boolean equals(Object objectToCompare) {
+        if (super.equals(objectToCompare) && objectToCompare instanceof ObjectAccessor) {
+            ObjectAccessor objectAccessor = (ObjectAccessor) objectToCompare;
+            
+            if (! valuesMatch(m_id, objectAccessor.getId())) {
+                return false;
+            }
+            
+            if (! valuesMatch(m_optional, objectAccessor.getOptional())) {
+                return false;
+            }
+
+            if (! valuesMatch(m_primaryKeyJoinColumns, objectAccessor.getPrimaryKeyJoinColumns())) {
+                return false;
+            }
+            
+            return valuesMatch(m_mapsId, objectAccessor.getMapsId());
+        }
+        
+        return false;
+    }
+    
     /**
      * INTERNAL:
      * Return true is this accessor is a derived id accessor.
@@ -167,7 +200,7 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
      * Used for OX mapping.
      */
     public Boolean getOptional() {
-        return m_isOptional;
+        return m_optional;
     }
     
     /**
@@ -237,13 +270,55 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
      */
     protected OneToOneMapping initOneToOneMapping() {
         OneToOneMapping mapping = new OneToOneMapping();
+        mapping.setIsOneToOneRelationship(true);
         mapping.setIsReadOnly(false);
-        mapping.setJoinFetch(getMappingJoinFetchType(getJoinFetch()));
         mapping.setIsOptional(isOptional());
         mapping.setAttributeName(getAttributeName());
         mapping.setReferenceClassName(getReferenceClassName());
         mapping.setDerivesId(derivesId());
         
+        // Process join fetch type.
+        processJoinFetch(getJoinFetch(), mapping);
+        
+        // Process the batch fetch if specified.
+        processBatchFetch(getBatchFetch(), mapping);
+        
+        // Process the orphanRemoval or PrivateOwned
+        processOrphanRemoval(mapping);
+        
+        // Process the indirection.
+        processIndirection(mapping);
+        
+        // Set the getter and setter methods if access is PROPERTY.
+        setAccessorMethods(mapping);
+        
+        // Process the cascade types.
+        processCascadeTypes(mapping);
+        
+        // Process a @ReturnInsert and @ReturnUpdate (to log a warning message)
+        processReturnInsertAndUpdate();
+        
+        return mapping;
+    }
+    
+    /**
+     * INTERNAL:
+     * Initialize a ManyToOneMapping.
+     */
+    protected ManyToOneMapping initManyToOneMapping() {
+        ManyToOneMapping mapping = new ManyToOneMapping();
+        mapping.setIsReadOnly(false);
+        mapping.setIsOptional(isOptional());
+        mapping.setAttributeName(getAttributeName());
+        mapping.setReferenceClassName(getReferenceClassName());
+        mapping.setDerivesId(derivesId());
+        
+        // Process join fetch type.
+        processJoinFetch(getJoinFetch(), mapping);
+        
+        // Process the batch fetch if specified.
+        processBatchFetch(getBatchFetch(), mapping);
+        
         // Process the orphanRemoval or PrivateOwned
         processOrphanRemoval(mapping);
         
@@ -285,7 +360,7 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
      * INTERNAL:
      */
     public boolean isOptional() {
-        return m_isOptional != null && m_isOptional;
+        return m_optional != null && m_optional;
     }
     
     /**
@@ -398,7 +473,7 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
             usesIndirection = false;
         }
         
-        if (usesIndirection && usesPropertyAccess(getDescriptor())) {
+        if (usesIndirection && usesPropertyAccess()) {
             mapping.setIndirectionPolicy(new WeavedObjectBasicIndirectionPolicy(getSetMethodName()));
         } else {
             mapping.setUsesIndirection(usesIndirection);
@@ -625,7 +700,7 @@ public abstract class ObjectAccessor extends RelationshipAccessor {
      * Used for OX mapping.
      */
     public void setOptional(Boolean isOptional) {
-        m_isOptional = isOptional;
+        m_optional = isOptional;
     }
     
     /**
