@@ -54,8 +54,6 @@ import org.xml.sax.EntityResolver;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
-import org.xml.sax.SAXNotRecognizedException;
-import org.xml.sax.SAXNotSupportedException;
 import org.eclipse.persistence.internal.oxm.record.XMLReader;
 
 /**
@@ -90,6 +88,13 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
             saxParserFactory = SAXParserFactory.newInstance();
             saxParserFactory.setNamespaceAware(true);
             saxParserFactory.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
+            try {
+                saxParserFactory.setFeature("http://java.sun.com/xml/schema/features/report-ignored-element-content-whitespace", true);
+            } catch(org.xml.sax.SAXNotRecognizedException ex) {
+                //ignore if the parser doesn't recognize or support this feature
+            } catch(org.xml.sax.SAXNotSupportedException ex) {
+            }
+            
             if(null != parserFeatures) {
             	for(Map.Entry<String, Boolean> parserFeature : parserFeatures.entrySet()) {
                     try {
@@ -206,9 +211,15 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
                 return unmarshal(domElement);
             }
             FileInputStream inputStream = new FileInputStream(file);
-            return unmarshal(inputStream);
+            try {
+                return unmarshal(inputStream);
+            } finally {
+                inputStream.close();
+            }
         } catch (FileNotFoundException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } catch (IOException e) {
+            throw XMLMarshalException.unmarshalException(e);
         }
     }
 
@@ -219,9 +230,15 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
                 return unmarshal(domElement, clazz);
             }
             FileInputStream inputStream = new FileInputStream(file);
-            return unmarshal(inputStream, clazz);
+            try {
+                return unmarshal(inputStream, clazz);
+            } finally {
+                inputStream.close();
+            }
         } catch (FileNotFoundException e) {
             throw XMLMarshalException.unmarshalException(e);
+        } catch (IOException e) {
+            throw XMLMarshalException.unmarshalException(e);
         }
     }
 
@@ -289,7 +306,7 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
         } else if(clazz == ClassConstants.OBJECT) {
     	    try{
                 SAXUnmarshallerHandler saxUnmarshallerHandler = new SAXUnmarshallerHandler(xmlUnmarshaller.getXMLContext());
-                saxUnmarshallerHandler.setXMLReader((XMLReader)xmlReader);
+                saxUnmarshallerHandler.setXMLReader(xmlReader);
                 saxUnmarshallerHandler.setUnmarshaller(xmlUnmarshaller);
                 saxUnmarshallerHandler.setKeepAsElementPolicy(UnmarshalKeepAsElementPolicy.KEEP_UNKNOWN_AS_ELEMENT);
                 xmlReader.setContentHandler(saxUnmarshallerHandler);
@@ -315,14 +332,8 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
             unmarshalRecord.setXMLReader(xmlReader);
             unmarshalRecord.setUnmarshaller(xmlUnmarshaller);
             xmlReader.setContentHandler(unmarshalRecord);
-            try {
-                unmarshalRecord.getXMLReader().setProperty("http://xml.org/sax/properties/lexical-handler", unmarshalRecord);
-            } catch (SAXNotRecognizedException ex) {
-            } catch (SAXNotSupportedException ex) {
-                //if lexical handling is not supported by this parser, just ignore. 
-            }
+            xmlReader.setLexicalHandler(unmarshalRecord);
             xmlReader.parse(inputSource);
-
         } catch (IOException e) {
             throw XMLMarshalException.unmarshalException(e);
         } catch (SAXException e) {
@@ -405,9 +416,8 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
             unmarshalRecord.setXMLReader(domReader);
             unmarshalRecord.setUnmarshaller(xmlUnmarshaller);
             domReader.setContentHandler(unmarshalRecord);
-            domReader.setProperty("http://xml.org/sax/properties/lexical-handler", unmarshalRecord);
+            domReader.setLexicalHandler(unmarshalRecord);
             domReader.parse(node);
-
         } catch (SAXException e) {
             throw convertSAXException(e);
         }
@@ -601,12 +611,7 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
             unmarshalRecord.setXMLReader(xmlReader);
             unmarshalRecord.setUnmarshaller(xmlUnmarshaller);
             xmlReader.setContentHandler(unmarshalRecord);
-            try {
-                unmarshalRecord.getXMLReader().setProperty("http://xml.org/sax/properties/lexical-handler", unmarshalRecord);
-            } catch (SAXNotRecognizedException ex) {
-            } catch (SAXNotSupportedException ex) {
-                //if lexical handling is not supported by this parser, just ignore. 
-            }
+            xmlReader.setLexicalHandler(unmarshalRecord);
             xmlReader.parse(systemId);
         } catch (IOException e) {
             throw XMLMarshalException.unmarshalException(e);
@@ -704,12 +709,7 @@ public class SAXUnmarshaller implements PlatformUnmarshaller {
             unmarshalRecord.setXMLReader(extendedXMLReader);
             unmarshalRecord.setUnmarshaller(xmlUnmarshaller);
             extendedXMLReader.setContentHandler(unmarshalRecord);
-            try {
-                extendedXMLReader.setProperty("http://xml.org/sax/properties/lexical-handler", unmarshalRecord);
-            } catch (SAXNotRecognizedException ex) {
-            } catch (SAXNotSupportedException ex) {
-                //if lexical handling is not supported by this parser, just ignore. 
-            }
+            extendedXMLReader.setLexicalHandler(unmarshalRecord);
             extendedXMLReader.parse(inputSource);
 
             // resolve mapping references
