@@ -26,7 +26,6 @@ import org.eclipse.persistence.internal.databaseaccess.*;
 import org.eclipse.persistence.internal.sessions.*;
 import org.eclipse.persistence.sessions.DatabaseRecord;
 import org.eclipse.persistence.tools.profiler.QueryMonitor;
-import org.eclipse.persistence.internal.identitymaps.CacheKey;
 import org.eclipse.persistence.expressions.*;
 import org.eclipse.persistence.queries.*;
 import org.eclipse.persistence.exceptions.*;
@@ -120,26 +119,22 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         
         ClassDescriptor descriptor = getDescriptor();
         List<DatabaseField> primaryKeyFields = descriptor.getPrimaryKeyFields();
-        Vector primaryKeyValues = new Vector(primaryKeyFields.size());
         
+        // Check that the query is by primary key.
         for (DatabaseField primaryKeyField : primaryKeyFields) {
-            if (queryFields.contains(primaryKeyField)) {
-                Object value = translationRow.get(primaryKeyField);
-            
-                if (value == null) {
-                    return null;
-                } else {
-                    primaryKeyValues.add(value);    
-                }
-            } else {
+            if (!queryFields.contains(primaryKeyField)) {
                 return null;
             }
         }
+        Object primaryKey = descriptor.getObjectBuilder().extractPrimaryKeyFromRow(translationRow, session);
+        if (primaryKey == null) {
+            return null;
+        }
         
         if (descriptor.shouldAcquireCascadedLocks()) {
-            return session.getIdentityMapAccessorInstance().getFromIdentityMapWithDeferredLock(primaryKeyValues, getReadObjectQuery().getReferenceClass(), false, descriptor);
+            return session.getIdentityMapAccessorInstance().getFromIdentityMapWithDeferredLock(primaryKey, getReadObjectQuery().getReferenceClass(), false, descriptor);
         } else {
-            return session.getIdentityMapAccessorInstance().getFromIdentityMap(primaryKeyValues, getReadObjectQuery().getReferenceClass(), false, descriptor);
+            return session.getIdentityMapAccessorInstance().getFromIdentityMap(primaryKey, getReadObjectQuery().getReferenceClass(), false, descriptor);
         }        
     }
 
@@ -258,6 +253,14 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         }
     }
 
+    /**
+     * Execute the call that was deferred to the commit manager.
+     * This is used to allow multiple table batching and deadlock avoidance.
+     */
+    public void executeDeferredCall(DatasourceCall call) {
+        // Do nothing by default.
+    }
+    
     /**
      * Check whether the object already exists on the database; then
      * perform an insert or update, as appropriate.
@@ -770,7 +773,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
 
         getDescriptor().getObjectBuilder().assignReturnRow(object, getSession(), row);
 
-        Vector primaryKeys = null;
+        Object primaryKey = null;
         if (isFirstCallForInsert) {
             AbstractRecord pkToModify = new DatabaseRecord();
             List primaryKeyFields = getDescriptor().getPrimaryKeyFields();
@@ -781,8 +784,8 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
                 }
             }
             if (!pkToModify.isEmpty()) {
-                primaryKeys = getDescriptor().getObjectBuilder().extractPrimaryKeyFromObject(object, getSession());
-                writeQuery.setPrimaryKey(primaryKeys);
+                primaryKey = getDescriptor().getObjectBuilder().extractPrimaryKeyFromObject(object, getSession());
+                writeQuery.setPrimaryKey(primaryKey);
                 // Now I need to update the row
                 getModifyRow().putAll(pkToModify);
                 getDescriptor().getObjectBuilder().addPrimaryKeyForNonDefaultTable(getModifyRow(), object, getSession());
@@ -797,8 +800,8 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             }
             if (objectChangeSet != null) {
                 updateChangeSet(getDescriptor(), objectChangeSet, row, object);
-                if (primaryKeys != null) {
-                    objectChangeSet.setCacheKey(new CacheKey(primaryKeys));
+                if (primaryKey != null) {
+                    objectChangeSet.setId(primaryKey);
                 }
             }
         }
@@ -918,7 +921,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
         if (sequenceValue == null) {
             return;
         }
-        Vector primaryKey = objectBuilder.extractPrimaryKeyFromObject(object, getSession());
+        Object primaryKey = objectBuilder.extractPrimaryKeyFromObject(object, getSession());
         writeQuery.setPrimaryKey(primaryKey);
         DatabaseField sequenceNumberField = descriptor.getSequenceNumberField();
         AbstractRecord modifyRow = getModifyRow();
@@ -935,7 +938,7 @@ public abstract class DatabaseQueryMechanism implements Cloneable, Serializable
             }
             if (objectChangeSet != null) {
                 updateChangeSet(descriptor, objectChangeSet, sequenceNumberField, object);
-                objectChangeSet.setCacheKey(new CacheKey(primaryKey));
+                objectChangeSet.setId(primaryKey);
             }
         }
     }
