@@ -48,7 +48,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Deferred lock the identity map for the object, this is used for avoiding deadlock
      * The return cacheKey should be used to release the deferred lock
      */
-    public CacheKey acquireDeferredLock(Vector primaryKey, Class javaClass, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey acquireDeferredLock(Object primaryKey, Class javaClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireDeferredLock(primaryKey, javaClass, descriptor);
         } else {
@@ -61,7 +62,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Provides access for setting a concurrency lock on an object in the IdentityMap.
      * called with true from the merge process, if true then the refresh will not refresh the object.
      */
-    public CacheKey acquireLock(Vector primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey acquireLock(Object primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireLock(primaryKey, domainClass, forMerge, descriptor);
         } else {
@@ -74,7 +76,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Provides access for setting a concurrency lock on an object in the IdentityMap.
      * called with true from the merge process, if true then the refresh will not refresh the object.
      */
-    public CacheKey acquireLockNoWait(Vector primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey acquireLockNoWait(Object primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireLockNoWait(primaryKey, domainClass, forMerge, descriptor);
         } else {
@@ -87,7 +90,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Provides access for setting a concurrency lock on an object in the IdentityMap.
      * called with true from the merge process, if true then the refresh will not refresh the object.
      */
-    public CacheKey acquireLockWithWait(Vector primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor, int wait) {
+    @Override
+    public CacheKey acquireLockWithWait(Object primaryKey, Class domainClass, boolean forMerge, ClassDescriptor descriptor, int wait) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireLockWithWait(primaryKey, domainClass, forMerge, descriptor, wait);
         } else {
@@ -101,7 +105,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * This will allow multiple users to read the same object but prevent writes to
      * the object while the read lock is held.
      */
-    public CacheKey acquireReadLockOnCacheKey(Vector primaryKey, Class domainClass, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey acquireReadLockOnCacheKey(Object primaryKey, Class domainClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireReadLockOnCacheKey(primaryKey, domainClass, descriptor);
         } else {
@@ -116,7 +121,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * the object while the read lock is held.
      * If no readlock can be acquired then do not wait but return null.
      */
-    public CacheKey acquireReadLockOnCacheKeyNoWait(Vector primaryKey, Class domainClass, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey acquireReadLockOnCacheKeyNoWait(Object primaryKey, Class domainClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().acquireReadLockOnCacheKeyNoWait(primaryKey, domainClass, descriptor);
         } else {
@@ -130,6 +136,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
     * By default concurrent reads and writes are allowed.
     * By write, unit of work merge is meant.
     */
+    @Override
     public boolean acquireWriteLock() {
         getIdentityMapManager().acquireWriteLock();
         // must lock the parents cache as well.
@@ -140,7 +147,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * ADVANCED:
      * Return if their is an object for the primary key.
      */
-    public boolean containsObjectInIdentityMap(Vector primaryKey, Class theClass, ClassDescriptor descriptor) {
+    @Override
+    public boolean containsObjectInIdentityMap(Object primaryKey, Class theClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             if (this.identityMapManager == null) {
                 return false;
@@ -155,6 +163,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * INTERNAL:
      * This method is used to get a list of those classes with IdentityMaps in the Session.
      */
+    @Override
     public Vector getClassesRegistered() {
         Vector results = getIdentityMapManager().getClassesRegistered();
         results.addAll(((IsolatedClientSession)session).getParent().getIdentityMapAccessorInstance().getClassesRegistered());
@@ -167,6 +176,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * If the expression is too complex an exception will be thrown.
      * Only return objects that are invalid in the cache if specified.
      */
+    @Override
     public Vector getAllFromIdentityMap(Expression selectionCriteria, Class theClass, Record translationRow, int valueHolderPolicy, boolean shouldReturnInvalidatedObjects) throws QueryException {
         if (session.getDescriptor(theClass).isIsolated()) {
             return getIdentityMapManager().getAllFromIdentityMap(selectionCriteria, theClass, translationRow, valueHolderPolicy, shouldReturnInvalidatedObjects);
@@ -181,7 +191,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * @param primaryKey the primary key of the cache key to be retrieved.
      * @param myClass the class of the cache key to be retrieved.
      */
-    public CacheKey getCacheKeyForObject(Vector primaryKey, Class myClass, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey getCacheKeyForObject(Object primaryKey, Class myClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             if (this.identityMapManager == null) {
                 return null;
@@ -198,7 +209,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * @param primaryKey the primary key of the cache key to be retrieved.
      * @param myClass the class of the cache key to be retrieved.
      */
-    public CacheKey getCacheKeyForObjectForLock(Vector primaryKey, Class myClass, ClassDescriptor descriptor) {
+    @Override
+    public CacheKey getCacheKeyForObjectForLock(Object primaryKey, Class myClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             if (this.identityMapManager == null) {
                 return null;
@@ -213,7 +225,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * ADVANCED:
      * Return the object from the identity with the primary and class.
      */
-    public Object getFromIdentityMap(Vector primaryKey, Class theClass, boolean shouldReturnInvalidatedObjects, ClassDescriptor descriptor) {
+    @Override
+    public Object getFromIdentityMap(Object primaryKey, Class theClass, boolean shouldReturnInvalidatedObjects, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             if (this.identityMapManager == null) {
                 return null;
@@ -230,6 +243,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * If the object is not found null is returned.
      * If the expression is too complex an exception will be thrown.
      */
+    @Override
     public Object getFromIdentityMap(Expression selectionCriteria, Class theClass, Record translationRow, int valueHolderPolicy, boolean conforming, boolean shouldReturnInvalidatedObjects, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().getFromIdentityMap(selectionCriteria, theClass, translationRow, valueHolderPolicy, conforming, shouldReturnInvalidatedObjects, descriptor);
@@ -243,7 +257,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Return the object from the identity with the primary and class.
      * Only return invalidated objects if requested
      */
-    public Object getFromIdentityMapWithDeferredLock(Vector primaryKey, Class theClass, boolean shouldReturnInvalidatedObjects, ClassDescriptor descriptor) {
+    @Override
+    public Object getFromIdentityMapWithDeferredLock(Object primaryKey, Class theClass, boolean shouldReturnInvalidatedObjects, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             if (this.identityMapManager == null) {
                 return null;
@@ -260,6 +275,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * This method should be used for all IdentityMapManager access since it may
      * be overridden in sub classes.
      */
+    @Override
     public IdentityMapManager getIdentityMapManager() {
         // PERF: Lazy init manager as normally isolated object are only read in the unit of work.
         if (this.identityMapManager == null) {
@@ -272,11 +288,12 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * INTERNAL:
      * Get the identity map for the given class from the IdentityMapManager
      */
-    public IdentityMap getIdentityMap(ClassDescriptor descriptor) {
+    @Override
+    public IdentityMap getIdentityMap(ClassDescriptor descriptor, boolean returnNullIfMissing) {
         if (descriptor.isIsolated()) {
-            return getIdentityMapManager().getIdentityMap(descriptor);
+            return getIdentityMapManager().getIdentityMap(descriptor, returnNullIfMissing);
         } else {
-            return ((IsolatedClientSession)session).getParent().getIdentityMapAccessorInstance().getIdentityMap(descriptor);
+            return ((IsolatedClientSession)session).getParent().getIdentityMapAccessorInstance().getIdentityMap(descriptor, returnNullIfMissing);
         }
     }
 
@@ -286,7 +303,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * values of the parameters to the query so different parameters will have
      * different cached results.
      */
-    public Object getQueryResult(ReadQuery query, Vector parameters, boolean checkExpiry) {
+    @Override
+    public Object getQueryResult(ReadQuery query, List parameters, boolean checkExpiry) {
         if (((IsolatedClientSession)session).isIsolatedQuery(query)) {
             return getIdentityMapManager().getQueryResult(query, parameters, checkExpiry);
         } else {
@@ -298,6 +316,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * INTERNAL:
      * get the session associated with this IdentityMapAccessor
      */
+    @Override
     public AbstractSession getSession() {
         return session;
     }
@@ -307,7 +326,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Get the wrapper object from the cache key associated with the given primary key,
      * this is used for EJB.
      */
-    public Object getWrapper(Vector primaryKey, Class theClass) {
+    @Override
+    public Object getWrapper(Object primaryKey, Class theClass) {
         if (session.getDescriptor(theClass).isIsolated()) {
             return getIdentityMapManager().getWrapper(primaryKey, theClass);
         } else {
@@ -329,7 +349,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * ADVANCED:
      * Extract the write lock value from the identity map.
      */
-    public Object getWriteLockValue(Vector primaryKey, Class theClass, ClassDescriptor descriptor) {
+    @Override
+    public Object getWriteLockValue(Object primaryKey, Class theClass, ClassDescriptor descriptor) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().getWriteLockValue(primaryKey, theClass, descriptor);
         } else {
@@ -346,6 +367,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * be maintained for any objects currently read in.  This should only be called
      * if the application knows that it no longer has references to object held in the cache.
      */
+    @Override
     public void initializeAllIdentityMaps() {
         super.initializeAllIdentityMaps();
         ((IsolatedClientSession)session).getParent().getIdentityMapAccessorInstance().initializeAllIdentityMaps();
@@ -358,6 +380,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Caution must be used in doing this to ensure that the objects within the identity map
      * are not referenced from other objects of other classes or from the application.
      */
+    @Override
     public void initializeIdentityMap(Class theClass) {
         getSession().log(SessionLog.FINER, SessionLog.CACHE, "initialize_identitymap", theClass);
         if (session.getDescriptor(theClass).isIsolated()) {
@@ -371,10 +394,11 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * PUBLIC:
      * Reset the entire local object cache.
      * This throws away any objects that have been read in.
-     * Extream caution should be used before doing this because object identity will no longer
+     * Extreme caution should be used before doing this because object identity will no longer
      * be maintained for any objects currently read in.  This should only be called
      * if the application knows that it no longer has references to object held in the cache.
      */
+    @Override
     public void initializeIdentityMaps() {
         getSession().log(SessionLog.FINER, SessionLog.CACHE, "initialize_identitymaps");
         getIdentityMapManager().initializeIdentityMaps();
@@ -387,7 +411,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Query results are cached based on the parameter values provided to the query
      * different parameter values access different caches.
      */
-    public void putQueryResult(ReadQuery query, Vector parameters, Object results) {
+    @Override
+    public void putQueryResult(ReadQuery query, List parameters, Object results) {
         if (((IsolatedClientSession)session).isIsolatedQuery(query)) {
             getIdentityMapManager().putQueryResult(query, parameters, results);
         } else {
@@ -400,6 +425,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Used to print all the objects in the identity map of the passed in class.
      * The output of this method will be logged to this session's SessionLog at SEVERE level.
      */
+    @Override
     public void printIdentityMap(Class businessClass) {
         if (getSession().shouldLog(SessionLog.SEVERE, SessionLog.CACHE)) {
             if (session.getDescriptor(businessClass).isIsolated()) {
@@ -415,6 +441,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Used to print all the objects in every identity map in this session.
      * The output of this method will be logged to this session's SessionLog at SEVERE level.
      */
+    @Override
     public void printIdentityMaps() {
         if (getSession().shouldLog(SessionLog.SEVERE, SessionLog.CACHE)) {
             getIdentityMapManager().printIdentityMaps();
@@ -427,6 +454,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Used to print all the locks in every identity map in this session.
      * The output of this method will be logged to this session's SessionLog at FINEST level.
      */
+    @Override
     public void printIdentityMapLocks() {
         if (getSession().shouldLog(SessionLog.FINEST, SessionLog.CACHE)) {
             getIdentityMapManager().printLocks();
@@ -440,7 +468,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * The object must always be registered with its version number if optimistic locking is used.
      * The readTime may also be included in the cache key as it is constructed.
      */
-    public CacheKey internalPutInIdentityMap(Object domainObject, Vector key, Object writeLockValue, long readTime, ClassDescriptor descriptor) {
+    public CacheKey internalPutInIdentityMap(Object domainObject, Object key, Object writeLockValue, long readTime, ClassDescriptor descriptor) {
         //no need to unwrap as the put will unwrap later anyway
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().putInIdentityMap(domainObject, key, writeLockValue, readTime, descriptor);
@@ -458,6 +486,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * @param translationRow Record
      * @param shouldInvalidateOnException boolean indicates weather to invalidate the object if conform threw exception.
      */
+    @Override
     public void invalidateObjects(Expression selectionCriteria, Class theClass, Record translationRow, boolean shouldInvalidateOnException) {
         if (session.getDescriptor(theClass).isIsolated()) {
             getIdentityMapManager().invalidateObjects(selectionCriteria, theClass, translationRow, shouldInvalidateOnException);
@@ -472,6 +501,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
     * By default concurrent reads and writes are allowed.
     * By write, unit of work merge is meant.
     */
+    @Override
     public void releaseWriteLock() {
         //release in the opposite order of the acquire
         ((IsolatedClientSession)session).getParent().getIdentityMapAccessorInstance().releaseWriteLock();
@@ -482,7 +512,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * ADVANCED:
      * Remove the object from the object cache.
      */
-    public Object removeFromIdentityMap(Vector key, Class theClass, ClassDescriptor descriptor, Object object) {
+    @Override
+    public Object removeFromIdentityMap(Object key, Class theClass, ClassDescriptor descriptor, Object object) {
         if (descriptor.isIsolated()) {
             return getIdentityMapManager().removeFromIdentityMap(key, theClass, descriptor, object);
         } else {
@@ -494,6 +525,7 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * INTERNAL:
      * Set the IdentityMapManager for this IdentityMapAccessor
      */
+    @Override
     public void setIdentityMapManager(IdentityMapManager identityMapManager) {
         this.identityMapManager = identityMapManager;
     }
@@ -503,7 +535,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * Update the wrapper object the cache key associated with the given primary key,
      * this is used for EJB.
      */
-    public void setWrapper(Vector primaryKey, Class theClass, Object wrapper) {
+    @Override
+    public void setWrapper(Object primaryKey, Class theClass, Object wrapper) {
         if (getSession().getDescriptor(theClass).isIsolated()) {
             getIdentityMapManager().setWrapper(primaryKey, theClass, wrapper);
         } else {
@@ -515,7 +548,8 @@ public class IsolatedClientSessionIdentityMapAccessor extends org.eclipse.persis
      * ADVANCED:
      * Update the write lock value in the identity map.
      */
-    public void updateWriteLockValue(Vector primaryKey, Class theClass, Object writeLockValue) {
+    @Override
+    public void updateWriteLockValue(Object primaryKey, Class theClass, Object writeLockValue) {
         if (getSession().getDescriptor(theClass).isIsolated()) {
             getIdentityMapManager().setWriteLockValue(primaryKey, theClass, writeLockValue);
         } else {
