@@ -15,8 +15,10 @@ package org.eclipse.persistence.queries;
 import java.util.*;
 import java.sql.*;
 import org.eclipse.persistence.internal.databaseaccess.*;
+import org.eclipse.persistence.internal.identitymaps.CacheId;
 import org.eclipse.persistence.internal.indirection.ProxyIndirectionPolicy;
 import org.eclipse.persistence.internal.descriptors.*;
+import org.eclipse.persistence.internal.queries.CallQueryMechanism;
 import org.eclipse.persistence.internal.sessions.remote.*;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
@@ -45,7 +47,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
     protected transient Object selectionObject;
 
     /** Key that can be used in place of a selection criteria. */
-    protected Vector selectionKey;
+    protected Object selectionId;
 
     /** Can be used to refresh a specific non-cached instance from the database. */
     protected boolean shouldLoadResultIntoSelectionObject = false;    
@@ -154,12 +156,16 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
      * It will cause the original to be cached in the query results if the query
      * is set to do so.
      */
-    public void cacheResult(Object unwrappedOriginal) {
-        Object cachableObject = unwrappedOriginal;
-        if (shouldUseWrapperPolicy()){
-            cachableObject = getSession().wrapObject(unwrappedOriginal);
+    public void cacheResult(Object object) {
+        Object cachableObject = object;
+        if (object == null) {
+            this.temporaryCachedQueryResults = InvalidObject.instance();
+        } else {
+            if (this.shouldUseWrapperPolicy) {
+                cachableObject = this.session.wrapObject(object);
+            }
+            this.temporaryCachedQueryResults = cachableObject;
         }
-        setTemporaryCachedQueryResults(cachableObject);
     }
 
 
@@ -224,7 +230,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
      * The cache check is done before the prepare as a hit will not require the work to be done.
      */
     protected Object checkEarlyReturnImpl(AbstractSession session, AbstractRecord translationRow) {
-        if (shouldCheckCache() && shouldMaintainCache() && (!shouldRefreshIdentityMapResult())
+        if (shouldCheckCache() && shouldMaintainCache() && (!shouldRefreshIdentityMapResult() && (!shouldRetrieveBypassCache()))
                 && (!(session.isRemoteSession() && (shouldRefreshRemoteIdentityMapResult() || this.descriptor.shouldDisableCacheHitsOnRemote())))
                 && (!(shouldCheckDescriptorForCacheUsage() && this.descriptor.shouldDisableCacheHits())) && (!this.descriptor.isDescriptorForInterface())) {
             Object cachedObject = getQueryMechanism().checkCacheForObject(translationRow, session);
@@ -292,7 +298,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
                         // PERF: the read-object query should always be static to ensure no regeneration of SQL.
                         if ((!hasJoining() || !getJoinedAttributeManager().hasJoinedAttributeExpressions()) && (!hasPartialAttributeExpressions()) && (!hasAsOfClause()) && (!hasNonDefaultFetchGroup()) && (getHintString() == null)
                                 && wasDefaultLockMode() && shouldIgnoreBindAllParameters() && (!hasFetchGroup()) && (getFetchGroupName() == null) && shouldUseDefaultFetchGroup()) {
-                            if ((getSelectionKey() != null) || (getSelectionObject() != null)) {// Must be primary key.
+                            if ((this.selectionId != null) || (this.selectionObject != null)) {// Must be primary key.
                                 setIsCustomQueryUsed(true);
                             } else {            
                                 if (getSelectionCriteria() != null) {
@@ -330,20 +336,22 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         // Must unwrap as the built object is always wrapped.
         // Note the object is unwrapped on the parent which it belongs to, as we
         // do not want to trigger a registration just yet.
-        Object implementation = null;
+        Object clone = null;
         if (buildDirectlyFromRows) {
-            implementation = result;
+            clone = buildObject((AbstractRecord)result);
         } else {
-            implementation = this.descriptor.getObjectBuilder().unwrapObject(result, unitOfWork.getParent());
+            clone = registerIndividualResult(
+                    this.descriptor.getObjectBuilder().unwrapObject(result, unitOfWork.getParent()),
+                    null, unitOfWork, null, null);
         }
-
-        if ((getSelectionCriteria() != null) && (getSelectionKey() == null) && (getSelectionObject() == null)) {
-            ExpressionBuilder builder = getSelectionCriteria().getBuilder();
+        Expression selectionCriteria = getSelectionCriteria();
+        if ((selectionCriteria != null) && (this.selectionId == null) && (this.selectionObject == null)) {
+            ExpressionBuilder builder = selectionCriteria.getBuilder();
             builder.setSession(unitOfWork.getRootSession(null));
             builder.setQueryClass(getReferenceClass());
         }
 
-        Object clone = conformIndividualResult(implementation, unitOfWork, databaseRow, getSelectionCriteria(), null, buildDirectlyFromRows);
+        clone = conformIndividualResult(clone, unitOfWork, databaseRow, selectionCriteria, null);
         if (clone == null) {
             return clone;
         }
@@ -410,10 +418,10 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         if (this.descriptor.isDescriptorForInterface()  || this.descriptor.hasTablePerClassPolicy()) {
             Object returnValue = this.descriptor.getInterfacePolicy().selectOneObjectUsingMultipleTableSubclassRead(this);
             
-            if (this.descriptor.hasTablePerClassPolicy() && returnValue == null) {
+            if (this.descriptor.hasTablePerClassPolicy() && (!this.descriptor.isAbstract()) && (returnValue == null)) {
                 // let it fall through to query the root.
             } else {
-                setExecutionTime(System.currentTimeMillis());
+                this.executionTime = System.currentTimeMillis();
                 return returnValue;
             }
         }
@@ -431,26 +439,28 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
             row = getQueryMechanism().selectOneRow();
         }
         
-        setExecutionTime(System.currentTimeMillis());
+        this.executionTime = System.currentTimeMillis();
         Object result = null;
-
-        if (session.isUnitOfWork()) {
-            result = registerResultInUnitOfWork(row, (UnitOfWorkImpl)session, getTranslationRow(), true);
-        } else {
-            if (row != null) {
+        if (row != null) {
+            if (session.isUnitOfWork()) {
+                result = registerResultInUnitOfWork(row, (UnitOfWorkImpl)session, this.translationRow, true);
+            } else {
                 result = buildObject(row);
             }
         }
-        if ((result == null) && shouldRefreshIdentityMapResult()) {
+        if ((result == null) && shouldCacheQueryResults()) {
+            cacheResult(null);
+        }
+        if ((result == null) && this.shouldRefreshIdentityMapResult) {
             // bug5955326, should invalidate the shared cached if refreshed object no longer exists.
-            if (getSelectionKey() != null) {
-                session.getParentIdentityMapSession(this, true, true).getIdentityMapAccessor().invalidateObject(getSelectionKey(), getReferenceClass());
-            } else if (getSelectionObject() != null) {
-                session.getParentIdentityMapSession(this, true, true).getIdentityMapAccessor().invalidateObject(getSelectionObject());
+            if (this.selectionId != null) {
+                session.getParentIdentityMapSession(this, true, true).getIdentityMapAccessor().invalidateObject(this.selectionId, this.referenceClass);
+            } else if (this.selectionObject != null) {
+                session.getParentIdentityMapSession(this, true, true).getIdentityMapAccessor().invalidateObject(this.selectionObject);
             }
         }                
 
-        if (shouldIncludeData()) {
+        if (this.shouldIncludeData) {
             ComplexQueryResult complexResult = new ComplexQueryResult();
             complexResult.setResult(result);
             complexResult.setData(row);
@@ -467,37 +477,38 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
      * @return object - the first object found or null if none.
      */
     protected Object executeObjectLevelReadQueryFromResultSet() throws DatabaseException {
-        UnitOfWorkImpl unitOfWork = (UnitOfWorkImpl)getSession();
-        DatabaseAccessor accessor = (DatabaseAccessor)unitOfWork.getAccessor();
-        DatabasePlatform platform = accessor.getPlatform();
-        DatabaseCall call = (DatabaseCall)getCall().clone();
-        call.setQuery(this);
-        AbstractRecord translationRow = getTranslationRow();
-        call.translate(getTranslationRow(), null, unitOfWork);
-        PreparedStatement statement = null;
-        ResultSet resultSet = null;
+        AbstractSession session = this.session;
+        DatabasePlatform platform = session.getPlatform();
+        DatabaseCall call = (DatabaseCall)((CallQueryMechanism)this.queryMechanism).getCall();
+        call.returnCursor();
+        call = this.queryMechanism.cursorSelectAllRows();
+        Statement statement = call.getStatement();
+        ResultSet resultSet = call.getResult();
+        DatabaseAccessor accessor = (DatabaseAccessor)this.accessor;
         boolean exceptionOccured = false;
         try {
-            accessor.incrementCallCount(unitOfWork);
-            statement = (PreparedStatement)call.prepareStatement(accessor, translationRow, unitOfWork);
-            resultSet = statement.executeQuery();
             if (!resultSet.next()) {
                 return null;
             }
             ResultSetMetaData metaData = resultSet.getMetaData();
-            return this.descriptor.getObjectBuilder().buildWorkingCopyCloneFromResultSet(this, null, resultSet, unitOfWork, accessor, metaData, platform);
+            return this.descriptor.getObjectBuilder().buildObjectFromResultSet(this, null, resultSet, session, accessor, metaData, platform);
         } catch (SQLException exception) {
             exceptionOccured = true;
             DatabaseException commException = accessor.processExceptionForCommError(session, exception, call);
-            if (commException != null) throw commException;
-            throw DatabaseException.sqlException(exception, call, accessor, unitOfWork, false);
+            if (commException != null) {
+                throw commException;
+            }
+            throw DatabaseException.sqlException(exception, call, accessor, session, false);
         } finally {
             try {
                 if (resultSet != null) {
                     resultSet.close();
                 }
                 if (statement != null) {
-                    accessor.releaseStatement(statement, call.getSQLString(), call, unitOfWork);
+                    accessor.releaseStatement(statement, call.getSQLString(), call, session);
+                }
+                if (accessor != null) {
+                    session.releaseReadConnection(accessor);
                 }
             } catch (SQLException exception) {
                 if (!exceptionOccured) {
@@ -505,7 +516,9 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
                     // if it is null we will be unable to check the connection for a comm error and
                     //therefore must return as if it was not a comm error.
                     DatabaseException commException = accessor.processExceptionForCommError(session, exception, call);
-                    if (commException != null) throw commException;
+                    if (commException != null) {
+                        throw commException;
+                    }
                     throw DatabaseException.sqlException(exception, call, accessor, session, false);
                 }
             }
@@ -533,9 +546,17 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
      * PUBLIC:
      * The primary key can be specified if used instead of an expression or selection object.
      * If composite the primary must be in the same order as defined in the descriptor.
+     * @Depreacted since EclipseLink 2.1, replaced by getSelectionId()
+     * @see #getSelectionId()
      */
+    @Deprecated
     public Vector getSelectionKey() {
-        return selectionKey;
+        if (this.selectionId instanceof CacheId) {
+            return new Vector(Arrays.asList(((CacheId)this.selectionId).getPrimaryKey()));
+        }
+        Vector primaryKey = new Vector(1);
+        primaryKey.add(this.selectionId);
+        return (Vector)selectionId;
 
     }
 
@@ -565,7 +586,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
      * Return if the query is by primary key.
      */
     public boolean isPrimaryKeyQuery() {
-        return (this.selectionKey != null) || (this.selectionObject != null);
+        return (this.selectionId != null) || (this.selectionObject != null);
     }
 
     /**
@@ -591,7 +612,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         super.copyFromQuery(query);
         if (query.isReadObjectQuery()) {
             ReadObjectQuery readQuery = (ReadObjectQuery)query;
-            this.selectionKey = readQuery.selectionKey;
+            this.selectionId = readQuery.selectionId;
             this.selectionObject = readQuery.selectionObject;
             this.shouldLoadResultIntoSelectionObject = readQuery.shouldLoadResultIntoSelectionObject;
         }
@@ -607,28 +628,41 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         }
         super.prepare();
         
-        if ((getSelectionKey() != null) || (getSelectionObject() != null)) {
+        if ((this.selectionId != null) || (this.selectionObject != null)) {
             // The expression is set in the prepare as params.
-            setSelectionCriteria(getDescriptor().getObjectBuilder().getPrimaryKeyExpression());
+            setSelectionCriteria(this.descriptor.getObjectBuilder().getPrimaryKeyExpression());
             setExpressionBuilder(getSelectionCriteria().getBuilder());
             extendPessimisticLockScope();
             // For bug 2989998 the translation row is required to be set at this point.
             if (!shouldPrepare()) {
-                if (getSelectionKey() != null) {
+                if (this.selectionId != null) {
                     // Row must come from the key.
-                    setTranslationRow(getDescriptor().getObjectBuilder().buildRowFromPrimaryKeyValues(getSelectionKey(), getSession()));
+                    setTranslationRow(this.descriptor.getObjectBuilder().buildRowFromPrimaryKeyValues(this.selectionId, this.session));
                 } else {//(getSelectionObject() != null)
-                    setTranslationRow(getDescriptor().getObjectBuilder().buildRowForTranslation(getSelectionObject(), getSession()));
+                    setTranslationRow(this.descriptor.getObjectBuilder().buildRowForTranslation(this.selectionObject, this.session));
                 }
             }
         }
 
-        if (getDescriptor().isDescriptorForInterface()) {
+        if (this.descriptor.isDescriptorForInterface()) {
             return;
         }
         
+        // PERF: Disable cache check if not a primary key query.
+        if (isExpressionQuery()) {
+            Expression selectionCriteria = getSelectionCriteria();
+            if (selectionCriteria != null) {
+                if (((this.cacheUsage == CheckCacheByPrimaryKey)
+                                && (!this.descriptor.getObjectBuilder().isPrimaryKeyExpression(false, selectionCriteria, this.session)))
+                        || ((this.cacheUsage == CheckCacheByExactPrimaryKey)
+                                && (!this.descriptor.getObjectBuilder().isPrimaryKeyExpression(true, selectionCriteria, this.session)))) {
+                    this.cacheUsage = DoNotCheckCache;
+                }
+            }
+        }
+        
         // If using 1-m joining select all rows.
-        if (hasJoining() && getJoinedAttributeManager().isToManyJoin()) {
+        if ((this.joinedAttributeManager != null) && this.joinedAttributeManager.isToManyJoin()) {
             getQueryMechanism().prepareSelectAllRows();
         } else {
             getQueryMechanism().prepareSelectOneRow();
@@ -637,25 +671,27 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
     
     /**
      * INTERNAL:
-     * Set the properties needed to be cascaded into the custom query inlucding the translation row.
+     * Set the properties needed to be cascaded into the custom query including the translation row.
+     * This is used only for primary key queries, as the descriptor query manager
+     * stores a predefined query for this query to avoid having to re-prepare and allow for customization.
      */
     protected void prepareCustomQuery(DatabaseQuery customQuery) {
         ReadObjectQuery customReadQuery = (ReadObjectQuery)customQuery;
-        customReadQuery.setShouldRefreshIdentityMapResult(shouldRefreshIdentityMapResult());
-        customReadQuery.setCascadePolicy(getCascadePolicy());
-        customReadQuery.setShouldMaintainCache(shouldMaintainCache());
-        customReadQuery.setShouldUseWrapperPolicy(shouldUseWrapperPolicy());
+        customReadQuery.shouldRefreshIdentityMapResult = this.shouldRefreshIdentityMapResult;
+        customReadQuery.cascadePolicy = this.cascadePolicy;
+        customReadQuery.shouldMaintainCache = this.shouldMaintainCache;
+        customReadQuery.shouldUseWrapperPolicy = this.shouldUseWrapperPolicy;
         // CR... was missing some values, execution could cause infinite loop.
-        customReadQuery.setQueryId(getQueryId());
-        customReadQuery.setExecutionTime(getExecutionTime());
-        customReadQuery.setShouldLoadResultIntoSelectionObject(shouldLoadResultIntoSelectionObject());
+        customReadQuery.queryId = this.queryId;
+        customReadQuery.executionTime = this.executionTime;
+        customReadQuery.shouldLoadResultIntoSelectionObject = this.shouldLoadResultIntoSelectionObject;
         AbstractRecord primaryKeyRow;
-        if (getSelectionObject() != null) {
+        if (this.selectionObject != null) {
             // CR#... Must also set the selection object as may be loading into the object (refresh)
-            customReadQuery.setSelectionObject(getSelectionObject());
+            customReadQuery.selectionObject = this.selectionObject;
             // The translation/primary key row will be set in prepareForExecution.
-        } else if (getSelectionKey() != null) {
-            customReadQuery.setSelectionKey(getSelectionKey());
+        } else if (this.selectionId != null) {
+            customReadQuery.selectionId = this.selectionId;
         } else {
             // The primary key row must be used.
             primaryKeyRow = customQuery.getDescriptor().getObjectBuilder().extractPrimaryKeyRowFromExpression(getSelectionCriteria(), customQuery.getTranslationRow(), customReadQuery.getSession());
@@ -672,12 +708,12 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
 
         // For bug 2989998 the translation row now sometimes set earlier in prepare.
         if (shouldPrepare()) {
-            if (getSelectionKey() != null) {
+            if (this.selectionId != null) {
                 // Row must come from the key.
-                setTranslationRow(this.descriptor.getObjectBuilder().buildRowFromPrimaryKeyValues(getSelectionKey(), getSession()));
-            } else if (getSelectionObject() != null) {
+                this.translationRow = this.descriptor.getObjectBuilder().buildRowFromPrimaryKeyValues(this.selectionId, getSession());
+            } else if (this.selectionObject != null) {
                 // The expression is set in the prepare as params.
-                setTranslationRow(this.descriptor.getObjectBuilder().buildRowForTranslation(getSelectionObject(), getSession()));
+                this.translationRow = this.descriptor.getObjectBuilder().buildRowForTranslation(this.selectionObject, getSession());
             }
         }
     }
@@ -723,7 +759,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         if (buildDirectlyFromRows) {
             clone = buildObject((AbstractRecord)result);
         } else {
-            clone = registerIndividualResult(result, unitOfWork, null);
+            clone = registerIndividualResult(result, null, unitOfWork, null, null);
         }
 
         if (shouldUseWrapperPolicy()) {
@@ -738,7 +774,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         // As the selection object is transient, compute the key.
         if (getSelectionObject() != null) {
             // Must be checked separately as the expression and row is not yet set.
-            setSelectionKey(getDescriptor().getObjectBuilder().extractPrimaryKeyFromObject(getSelectionObject(), session));
+            setSelectionId(getDescriptor().getObjectBuilder().extractPrimaryKeyFromObject(getSelectionObject(), session));
         }
 
         Object cacheHit = checkEarlyReturn(getSession(), getTranslationRow());
@@ -757,21 +793,43 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         return controller.replaceValueHoldersIn(object);
     }
 
+    /**
+     * PUBLIC:
+     * Return Id of the object to be selected by the query.
+     */
+    public Object getSelectionId() {
+        return this.selectionId;
+    }
+
+    /**
+     * PUBLIC:
+     * The Id of the object to be selected by the query.
+     * This will generate a query by primary key.
+     * This can be used instead of an Expression, SQL, or JPQL, or example object.
+     * The Id is the Id value for a singleton primary key,
+     * for a composite it is an instance of CacheId.
+     * @see CacheId
+     */
+    public void setSelectionId(Object id) {
+        this.selectionId = id;
+    }
+
     /**
      * PUBLIC:
      * The primary key can be specified if used instead of an expression or selection object.
      * If composite the primary must be in the same order as defined in the descriptor.
+     * @depreacted since EclipseLink 2.1, replaced by setSelectionId(Object)
+     * @see #setSelectionId(Object)
      */
+    @Deprecated
     public void setSelectionKey(List selectionKey) {
         if (selectionKey == null) {
-            this.selectionKey = null;
-        } else if (selectionKey instanceof NonSynchronizedVector) {
-            this.selectionKey = (Vector)selectionKey;
+            this.selectionId = null;
+        } else if (selectionKey.size() == 1) {
+            this.selectionId = selectionKey.get(0);            
         } else {
-            this.selectionKey = new NonSynchronizedVector(selectionKey);            
+            this.selectionId = new CacheId(selectionKey.toArray());
         }
-        // setIsPrepared(false); PERF: This cause the findByPrimaryKey query to unprepare,
-        // Since this is an argument, and not a query modifier do not unprepare.
     }
 
     /**
@@ -787,7 +845,7 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
         if (selectionObject == null) {
             throw QueryException.selectionObjectCannotBeNull(this);
         }
-        setSelectionKey(null);
+        setSelectionId(null);
         // Check if the query needs to be unprepared.
         if ((this.selectionObject == null) || (this.selectionObject.getClass() != selectionObject.getClass())) {
             setIsPrepared(false);
@@ -811,20 +869,14 @@ public class ReadObjectQuery extends ObjectLevelReadQuery {
     /**
      * PUBLIC:
      * The primary key can be specified if used instead of an expression or selection object.
+     * @Depreacted since EclipseLink 2.1, replaced by setSelectionId(Object)
+     * @see #setSelectionId(Object)
      */
+    @Deprecated
     public void setSingletonSelectionKey(Object selectionKey) {
-        Vector key = new Vector();
-        key.addElement(selectionKey);
+        Vector key = new NonSynchronizedVector();
+        key.add(selectionKey);
         setSelectionKey(key);
-
-    }
-
-    /**
-     * PUBLIC:
-     * Return if the cache should be checked.
-     */
-    public boolean shouldCheckCache() {
-        return this.cacheUsage != DoNotCheckCache;
     }
 
     /**
